/*

  TM1638 <stroke>library</stroke> sketch
  This actually it's not the library a sketch, which you have to load together
  with your usual setup() & loop() sketch. 
  
  Martin Hubacek
  http://www.martinhubacek.cz
  
  Change your pin numbers below

*/

#define TM1638_DATA 2
#define TM1638_CLK 3
#define TM1638_STROBE 4


unsigned char TM1638_numbers[]={
  0x3F,0x06,0x5B,0x4F,0x66,0x6D,0x7D,0x07,
  0x7F,0x6F,0x77,0x7C,0x39,0x5E,0x79,0x71};

// Display own segments on the display
// value - array of unsigned char of length 8, each byte is one digit
void TM1638_write_display(unsigned char *values)
{
  unsigned char i;

  for(i=0; i < 8; i++)
    TM1638_write_data(i << 1, values[i]);
}

// Display numbers on the display, integers 0-9 and A-F
// value - array of unsigned char of length 8, each byte is one digit
void TM1638_write_numbers(unsigned char *values)
{
  unsigned char i;

  for(i=0; i < 8; i++)
    TM1638_write_data(i << 1, TM1638_numbers[values[i]]);
}

// LEDs - 16bit number, lower 8bits are for red LEDs, higher 8 bits for green LEDs
void TM1638_write_LED(unsigned short LEDs)
{
  unsigned char i, val;
  for(i=0;i<8;i++)
  {
    val = 0;

    if(LEDs & (1 << i))
      val |= 1;
    if(LEDs & (1 << (i + 8)))
      val |= 2;  

    TM1638_write_data(2*i+1,val);
  }
}

void TM1638_write(unsigned char	data)
{
  unsigned char i;
  for(i=0; i<8; i++)
  {
    digitalWrite(TM1638_CLK, 0);
    if(data & 0x01)
      digitalWrite(TM1638_DATA, 1);
    else
      digitalWrite(TM1638_DATA, 0);
    data >>= 1;
    digitalWrite(TM1638_CLK, 1);
  }
}


void TM1638_cmd(unsigned char cmd)
{
  digitalWrite(TM1638_STROBE, 0);
  TM1638_write(cmd);
  digitalWrite(TM1638_STROBE, 1);
}

void TM1638_write_data(unsigned char add,unsigned char DATA)
{
  TM1638_cmd(0x44);
  digitalWrite(TM1638_STROBE, 0);
  TM1638_write(0xc0 | add);
  TM1638_write(DATA);
  digitalWrite(TM1638_STROBE, 1);
}



void TM1638_init()
{
  digitalWrite(TM1638_STROBE, 1);
  digitalWrite(TM1638_CLK, 1);

  pinMode(TM1638_CLK, OUTPUT);
  pinMode(TM1638_DATA, OUTPUT);
  pinMode(TM1638_STROBE, OUTPUT);

  unsigned char i;
  TM1638_cmd(0x8b);       
  TM1638_cmd(0x40);       
  digitalWrite(TM1638_STROBE, 0);
  TM1638_write(0xc0);    

  for(i=0;i<16;i++)	   
    TM1638_write(0x00);

  digitalWrite(TM1638_STROBE, 1);
}


unsigned char TM1638_read()
{
  unsigned char i;
  unsigned char temp=0;

  // Pull-up on
  pinMode(TM1638_DATA, INPUT);
  digitalWrite(TM1638_DATA, HIGH);        

  for(i=0;i<8;i++)
  {
    temp>>=1;
    digitalWrite(TM1638_CLK, LOW);        
    if(digitalRead(TM1638_DATA))
      temp|=0x80;
    digitalWrite(TM1638_CLK, HIGH);        
  }

  // Pull-up off
  pinMode(TM1638_DATA, OUTPUT);
  digitalWrite(TM1638_DATA, LOW); 

  return temp;
}


unsigned char TM1638_read_key(void)
{
  unsigned char c[4],i,key_value=0;
  digitalWrite(TM1638_STROBE, LOW);  
  TM1638_write(0x42);		
  for(i=0;i<4;i++)		
    c[i]=TM1638_read();
  digitalWrite(TM1638_STROBE, HIGH);

  for(i=0;i<4;i++)
    key_value|=c[i]<<i;

  for(i=0;i<8;i++)
    if((0x01<<i)==key_value)
      break;
  return i;
}
